@@ -1,11 +1,13 @@
 mod completion_provider;
 mod entry_view_state;
 mod message_editor;
+mod mode_selector;
 mod model_selector;
 mod model_selector_popover;
 mod thread_history;
 mod thread_view;
 
+pub use mode_selector::ModeSelector;
 pub use model_selector::AcpModelSelector;
 pub use model_selector_popover::AcpModelSelectorPopover;
 pub use thread_history::*;
