@@ -71,7 +71,7 @@ impl AcpModelPickerDelegate {
                     let (models, selected_model) = futures::join!(models_task, selected_model_task);
 
                     this.update_in(cx, |this, window, cx| {
-                        this.delegate.models = models.log_err();
+                        this.delegate.models = models.ok();
                         this.delegate.selected_model = selected_model.ok();
                         this.refresh(window, cx)
                     })
@@ -141,11 +141,6 @@ impl PickerDelegate for AcpModelPickerDelegate {
         cx.spawn_in(window, async move |this, cx| {
             let filtered_models = match this
                 .read_with(cx, |this, cx| {
-                    if let Some(models) = this.delegate.models.as_ref() {
-                        log::debug!("Filtering {} models.", models.len());
-                    } else {
-                        log::debug!("No models available.");
-                    }
                     this.delegate.models.clone().map(move |models| {
                         fuzzy_search(models, query, cx.background_executor().clone())
                     })
@@ -157,8 +152,6 @@ impl PickerDelegate for AcpModelPickerDelegate {
                 None => AgentModelList::Flat(vec![]),
             };
 
-            log::debug!("Filtered models. {} available.", filtered_models.len());
-
             this.update_in(cx, |this, window, cx| {
                 this.delegate.filtered_entries =
                     info_list_to_picker_entries(filtered_models).collect();
@@ -199,8 +192,10 @@ impl PickerDelegate for AcpModelPickerDelegate {
         }
     }
 
-    fn dismissed(&mut self, _: &mut Window, cx: &mut Context<Picker<Self>>) {
-        cx.emit(DismissEvent);
+    fn dismissed(&mut self, window: &mut Window, cx: &mut Context<Picker<Self>>) {
+        cx.defer_in(window, |picker, window, cx| {
+            picker.set_query("", window, cx);
+        });
     }
 
     fn render_match(
