@@ -166,7 +166,7 @@ impl SlashCommand for ExtensionSlashCommand {
                     .collect(),
                 run_commands_in_text: false,
             }
-            .to_event_stream())
+            .into_event_stream())
         })
     }
 }
