@@ -24,16 +24,16 @@ pub fn adapt_schema_to_format(
 fn preprocess_json_schema(json: &mut Value) -> Result<()> {
     // `additionalProperties` defaults to `false` unless explicitly specified.
     // This prevents models from hallucinating tool parameters.
-    if let Value::Object(obj) = json {
-        if matches!(obj.get("type"), Some(Value::String(s)) if s == "object") {
-            if !obj.contains_key("additionalProperties") {
-                obj.insert("additionalProperties".to_string(), Value::Bool(false));
-            }
+    if let Value::Object(obj) = json
+        && matches!(obj.get("type"), Some(Value::String(s)) if s == "object")
+    {
+        if !obj.contains_key("additionalProperties") {
+            obj.insert("additionalProperties".to_string(), Value::Bool(false));
+        }
 
-            // OpenAI API requires non-missing `properties`
-            if !obj.contains_key("properties") {
-                obj.insert("properties".to_string(), Value::Object(Default::default()));
-            }
+        // OpenAI API requires non-missing `properties`
+        if !obj.contains_key("properties") {
+            obj.insert("properties".to_string(), Value::Object(Default::default()));
         }
     }
     Ok(())
@@ -59,10 +59,10 @@ fn adapt_to_json_schema_subset(json: &mut Value) -> Result<()> {
             ("optional", |value| value.is_boolean()),
         ];
         for (key, predicate) in KEYS_TO_REMOVE {
-            if let Some(value) = obj.get(key) {
-                if predicate(value) {
-                    obj.remove(key);
-                }
+            if let Some(value) = obj.get(key)
+                && predicate(value)
+            {
+                obj.remove(key);
             }
         }
 
@@ -77,12 +77,12 @@ fn adapt_to_json_schema_subset(json: &mut Value) -> Result<()> {
         }
 
         // Handle oneOf -> anyOf conversion
-        if let Some(subschemas) = obj.get_mut("oneOf") {
-            if subschemas.is_array() {
-                let subschemas_clone = subschemas.clone();
-                obj.remove("oneOf");
-                obj.insert("anyOf".to_string(), subschemas_clone);
-            }
+        if let Some(subschemas) = obj.get_mut("oneOf")
+            && subschemas.is_array()
+        {
+            let subschemas_clone = subschemas.clone();
+            obj.remove("oneOf");
+            obj.insert("anyOf".to_string(), subschemas_clone);
         }
 
         // Recursively process all nested objects and arrays
