@@ -156,13 +156,13 @@ fn resolve_context_server_tool_name_conflicts(
 
     if duplicated_tool_names.is_empty() {
         return context_server_tools
-            .into_iter()
+            .iter()
             .map(|tool| (resolve_tool_name(tool).into(), tool.clone()))
             .collect();
     }
 
     context_server_tools
-        .into_iter()
+        .iter()
         .filter_map(|tool| {
             let mut tool_name = resolve_tool_name(tool);
             if !duplicated_tool_names.contains(&tool_name) {
