@@ -82,7 +82,7 @@ impl ChannelBuffer {
                 collaborators: Default::default(),
                 acknowledge_task: None,
                 channel_id: channel.id,
-                subscription: Some(subscription.set_entity(&cx.entity(), &mut cx.to_async())),
+                subscription: Some(subscription.set_entity(&cx.entity(), &cx.to_async())),
                 user_store,
                 channel_store,
             };
@@ -110,7 +110,7 @@ impl ChannelBuffer {
             let Ok(subscription) = self.client.subscribe_to_entity(self.channel_id.0) else {
                 return;
             };
-            self.subscription = Some(subscription.set_entity(&cx.entity(), &mut cx.to_async()));
+            self.subscription = Some(subscription.set_entity(&cx.entity(), &cx.to_async()));
             cx.emit(ChannelBufferEvent::Connected);
         }
     }
@@ -135,7 +135,7 @@ impl ChannelBuffer {
             }
         }
 
-        for (_, old_collaborator) in &self.collaborators {
+        for old_collaborator in self.collaborators.values() {
             if !new_collaborators.contains_key(&old_collaborator.peer_id) {
                 self.buffer.update(cx, |buffer, cx| {
                     buffer.remove_peer(old_collaborator.replica_id, cx)
@@ -191,12 +191,11 @@ impl ChannelBuffer {
                 operation,
                 is_local: true,
             } => {
-                if *ZED_ALWAYS_ACTIVE {
-                    if let language::Operation::UpdateSelections { selections, .. } = operation {
-                        if selections.is_empty() {
-                            return;
-                        }
-                    }
+                if *ZED_ALWAYS_ACTIVE
+                    && let language::Operation::UpdateSelections { selections, .. } = operation
+                    && selections.is_empty()
+                {
+                    return;
                 }
                 let operation = language::proto::serialize_operation(operation);
                 self.client
