@@ -39,7 +39,7 @@ pub const EDIT_PREDICTIONS_RESOURCE_HEADER_VALUE: &str = "edit_predictions";
 /// The name of the header used to indicate that the maximum number of consecutive tool uses has been reached.
 pub const TOOL_USE_LIMIT_REACHED_HEADER_NAME: &str = "x-zed-tool-use-limit-reached";
 
-/// The name of the header used to indicate the the minimum required Zed version.
+/// The name of the header used to indicate the minimum required Zed version.
 ///
 /// This can be used to force a Zed upgrade in order to continue communicating
 /// with the LLM service.
@@ -74,9 +74,21 @@ impl FromStr for UsageLimit {
     }
 }
 
+#[derive(Debug, Clone, Copy, PartialEq)]
+pub enum Plan {
+    V1(PlanV1),
+    V2(PlanV2),
+}
+
+impl Plan {
+    pub fn is_v2(&self) -> bool {
+        matches!(self, Self::V2(_))
+    }
+}
+
 #[derive(Debug, Clone, Copy, Default, PartialEq, Serialize, Deserialize)]
 #[serde(rename_all = "snake_case")]
-pub enum Plan {
+pub enum PlanV1 {
     #[default]
     #[serde(alias = "Free")]
     ZedFree,
@@ -86,40 +98,36 @@ pub enum Plan {
     ZedProTrial,
 }
 
-impl Plan {
-    pub fn as_str(&self) -> &'static str {
-        match self {
-            Plan::ZedFree => "zed_free",
-            Plan::ZedPro => "zed_pro",
-            Plan::ZedProTrial => "zed_pro_trial",
-        }
-    }
-
-    pub fn model_requests_limit(&self) -> UsageLimit {
-        match self {
-            Plan::ZedPro => UsageLimit::Limited(500),
-            Plan::ZedProTrial => UsageLimit::Limited(150),
-            Plan::ZedFree => UsageLimit::Limited(50),
-        }
-    }
-
-    pub fn edit_predictions_limit(&self) -> UsageLimit {
-        match self {
-            Plan::ZedPro => UsageLimit::Unlimited,
-            Plan::ZedProTrial => UsageLimit::Unlimited,
-            Plan::ZedFree => UsageLimit::Limited(2_000),
-        }
-    }
-}
-
-impl FromStr for Plan {
+impl FromStr for PlanV1 {
     type Err = anyhow::Error;
 
     fn from_str(value: &str) -> Result<Self, Self::Err> {
         match value {
-            "zed_free" => Ok(Plan::ZedFree),
-            "zed_pro" => Ok(Plan::ZedPro),
-            "zed_pro_trial" => Ok(Plan::ZedProTrial),
+            "zed_free" => Ok(Self::ZedFree),
+            "zed_pro" => Ok(Self::ZedPro),
+            "zed_pro_trial" => Ok(Self::ZedProTrial),
+            plan => Err(anyhow::anyhow!("invalid plan: {plan:?}")),
+        }
+    }
+}
+
+#[derive(Debug, Clone, Copy, Default, PartialEq, Serialize, Deserialize)]
+#[serde(rename_all = "snake_case")]
+pub enum PlanV2 {
+    #[default]
+    ZedFree,
+    ZedPro,
+    ZedProTrial,
+}
+
+impl FromStr for PlanV2 {
+    type Err = anyhow::Error;
+
+    fn from_str(value: &str) -> Result<Self, Self::Err> {
+        match value {
+            "zed_free" => Ok(Self::ZedFree),
+            "zed_pro" => Ok(Self::ZedPro),
+            "zed_pro_trial" => Ok(Self::ZedProTrial),
             plan => Err(anyhow::anyhow!("invalid plan: {plan:?}")),
         }
     }
@@ -313,14 +321,14 @@ pub struct LanguageModel {
 #[derive(Debug, Serialize, Deserialize)]
 pub struct ListModelsResponse {
     pub models: Vec<LanguageModel>,
-    pub default_model: LanguageModelId,
-    pub default_fast_model: LanguageModelId,
+    pub default_model: Option<LanguageModelId>,
+    pub default_fast_model: Option<LanguageModelId>,
     pub recommended_models: Vec<LanguageModelId>,
 }
 
 #[derive(Debug, Serialize, Deserialize)]
 pub struct GetSubscriptionResponse {
-    pub plan: Plan,
+    pub plan: PlanV1,
     pub usage: Option<CurrentUsage>,
 }
 
@@ -344,27 +352,39 @@ mod tests {
     use super::*;
 
     #[test]
-    fn test_plan_deserialize_snake_case() {
-        let plan = serde_json::from_value::<Plan>(json!("zed_free")).unwrap();
-        assert_eq!(plan, Plan::ZedFree);
+    fn test_plan_v1_deserialize_snake_case() {
+        let plan = serde_json::from_value::<PlanV1>(json!("zed_free")).unwrap();
+        assert_eq!(plan, PlanV1::ZedFree);
 
-        let plan = serde_json::from_value::<Plan>(json!("zed_pro")).unwrap();
-        assert_eq!(plan, Plan::ZedPro);
+        let plan = serde_json::from_value::<PlanV1>(json!("zed_pro")).unwrap();
+        assert_eq!(plan, PlanV1::ZedPro);
 
-        let plan = serde_json::from_value::<Plan>(json!("zed_pro_trial")).unwrap();
-        assert_eq!(plan, Plan::ZedProTrial);
+        let plan = serde_json::from_value::<PlanV1>(json!("zed_pro_trial")).unwrap();
+        assert_eq!(plan, PlanV1::ZedProTrial);
     }
 
     #[test]
-    fn test_plan_deserialize_aliases() {
-        let plan = serde_json::from_value::<Plan>(json!("Free")).unwrap();
-        assert_eq!(plan, Plan::ZedFree);
+    fn test_plan_v1_deserialize_aliases() {
+        let plan = serde_json::from_value::<PlanV1>(json!("Free")).unwrap();
+        assert_eq!(plan, PlanV1::ZedFree);
 
-        let plan = serde_json::from_value::<Plan>(json!("ZedPro")).unwrap();
-        assert_eq!(plan, Plan::ZedPro);
+        let plan = serde_json::from_value::<PlanV1>(json!("ZedPro")).unwrap();
+        assert_eq!(plan, PlanV1::ZedPro);
 
-        let plan = serde_json::from_value::<Plan>(json!("ZedProTrial")).unwrap();
-        assert_eq!(plan, Plan::ZedProTrial);
+        let plan = serde_json::from_value::<PlanV1>(json!("ZedProTrial")).unwrap();
+        assert_eq!(plan, PlanV1::ZedProTrial);
+    }
+
+    #[test]
+    fn test_plan_v2_deserialize_snake_case() {
+        let plan = serde_json::from_value::<PlanV2>(json!("zed_free")).unwrap();
+        assert_eq!(plan, PlanV2::ZedFree);
+
+        let plan = serde_json::from_value::<PlanV2>(json!("zed_pro")).unwrap();
+        assert_eq!(plan, PlanV2::ZedPro);
+
+        let plan = serde_json::from_value::<PlanV2>(json!("zed_pro_trial")).unwrap();
+        assert_eq!(plan, PlanV2::ZedProTrial);
     }
 
     #[test]
