@@ -18,9 +18,7 @@ use workspace::{Workspace, dock::Panel};
 use super::{TestClient, TestServer};
 
 pub fn init_test(cx: &mut gpui::TestAppContext) {
-    if std::env::var("RUST_LOG").is_ok() {
-        env_logger::try_init().ok();
-    }
+    zlog::init_test();
 
     cx.update(|cx| {
         theme::init(theme::LoadThemes::JustBase, cx);
