@@ -62,12 +62,6 @@ impl CopilotChatConfiguration {
     }
 }
 
-// Copilot's base model; defined by Microsoft in premium requests table
-// This will be moved to the front of the Copilot model list, and will be used for
-// 'fast' requests (e.g. title generation)
-// https://docs.github.com/en/copilot/managing-copilot/monitoring-usage-and-entitlements/about-premium-requests
-const DEFAULT_MODEL_ID: &str = "gpt-4.1";
-
 #[derive(Clone, Copy, Serialize, Deserialize, Debug, Eq, PartialEq)]
 #[serde(rename_all = "lowercase")]
 pub enum Role {
@@ -101,22 +95,41 @@ where
     Ok(models)
 }
 
-#[derive(Clone, Serialize, Deserialize, Debug, Eq, PartialEq)]
+#[derive(Clone, Serialize, Deserialize, Debug, PartialEq)]
 pub struct Model {
+    billing: ModelBilling,
     capabilities: ModelCapabilities,
     id: String,
     name: String,
     policy: Option<ModelPolicy>,
     vendor: ModelVendor,
+    is_chat_default: bool,
+    // The model with this value true is selected by VSCode copilot if a premium request limit is
+    // reached. Zed does not currently implement this behaviour
+    is_chat_fallback: bool,
     model_picker_enabled: bool,
 }
 
+#[derive(Clone, Serialize, Deserialize, Debug, PartialEq)]
+struct ModelBilling {
+    is_premium: bool,
+    multiplier: f64,
+    // List of plans a model is restricted to
+    // Field is not present if a model is available for all plans
+    #[serde(default)]
+    restricted_to: Option<Vec<String>>,
+}
+
 #[derive(Clone, Serialize, Deserialize, Debug, Eq, PartialEq)]
 struct ModelCapabilities {
     family: String,
     #[serde(default)]
     limits: ModelLimits,
     supports: ModelSupportedFeatures,
+    #[serde(rename = "type")]
+    model_type: String,
+    #[serde(default)]
+    tokenizer: Option<String>,
 }
 
 #[derive(Default, Clone, Serialize, Deserialize, Debug, Eq, PartialEq)]
@@ -155,6 +168,9 @@ pub enum ModelVendor {
     Anthropic,
     #[serde(rename = "xAI")]
     XAI,
+    /// Unknown vendor that we don't explicitly support yet
+    #[serde(other)]
+    Unknown,
 }
 
 #[derive(Serialize, Deserialize, Debug, Eq, PartialEq, Clone)]
@@ -203,6 +219,10 @@ impl Model {
     pub fn supports_parallel_tool_calls(&self) -> bool {
         self.capabilities.supports.parallel_tool_calls
     }
+
+    pub fn tokenizer(&self) -> Option<&str> {
+        self.capabilities.tokenizer.as_deref()
+    }
 }
 
 #[derive(Serialize, Deserialize)]
@@ -604,6 +624,7 @@ async fn get_models(
         .into_iter()
         .filter(|model| {
             model.model_picker_enabled
+                && model.capabilities.model_type.as_str() == "chat"
                 && model
                     .policy
                     .as_ref()
@@ -612,9 +633,7 @@ async fn get_models(
         .dedup_by(|a, b| a.capabilities.family == b.capabilities.family)
         .collect();
 
-    if let Some(default_model_position) =
-        models.iter().position(|model| model.id == DEFAULT_MODEL_ID)
-    {
+    if let Some(default_model_position) = models.iter().position(|model| model.is_chat_default) {
         let default_model = models.remove(default_model_position);
         models.insert(0, default_model);
     }
@@ -632,7 +651,9 @@ async fn request_models(
         .uri(models_url.as_ref())
         .header("Authorization", format!("Bearer {}", api_token))
         .header("Content-Type", "application/json")
-        .header("Copilot-Integration-Id", "vscode-chat");
+        .header("Copilot-Integration-Id", "vscode-chat")
+        .header("Editor-Version", "vscode/1.103.2")
+        .header("x-github-api-version", "2025-05-01");
 
     let request = request_builder.body(AsyncBody::empty())?;
 
@@ -803,6 +824,10 @@ mod tests {
         let json = r#"{
               "data": [
                 {
+                  "billing": {
+                    "is_premium": false,
+                    "multiplier": 0
+                  },
                   "capabilities": {
                     "family": "gpt-4",
                     "limits": {
@@ -816,6 +841,8 @@ mod tests {
                     "type": "chat"
                   },
                   "id": "gpt-4",
+                  "is_chat_default": false,
+                  "is_chat_fallback": false,
                   "model_picker_enabled": false,
                   "name": "GPT 4",
                   "object": "model",
@@ -827,6 +854,16 @@ mod tests {
                     "some-unknown-field": 123
                 },
                 {
+                  "billing": {
+                    "is_premium": true,
+                    "multiplier": 1,
+                    "restricted_to": [
+                      "pro",
+                      "pro_plus",
+                      "business",
+                      "enterprise"
+                    ]
+                  },
                   "capabilities": {
                     "family": "claude-3.7-sonnet",
                     "limits": {
@@ -850,6 +887,8 @@ mod tests {
                     "type": "chat"
                   },
                   "id": "claude-3.7-sonnet",
+                  "is_chat_default": false,
+                  "is_chat_fallback": false,
                   "model_picker_enabled": true,
                   "name": "Claude 3.7 Sonnet",
                   "object": "model",
@@ -871,4 +910,45 @@ mod tests {
         assert_eq!(schema.data[0].id, "gpt-4");
         assert_eq!(schema.data[1].id, "claude-3.7-sonnet");
     }
+
+    #[test]
+    fn test_unknown_vendor_resilience() {
+        let json = r#"{
+              "data": [
+                {
+                  "billing": {
+                    "is_premium": false,
+                    "multiplier": 1
+                  },
+                  "capabilities": {
+                    "family": "future-model",
+                    "limits": {
+                      "max_context_window_tokens": 128000,
+                      "max_output_tokens": 8192,
+                      "max_prompt_tokens": 120000
+                    },
+                    "object": "model_capabilities",
+                    "supports": { "streaming": true, "tool_calls": true },
+                    "type": "chat"
+                  },
+                  "id": "future-model-v1",
+                  "is_chat_default": false,
+                  "is_chat_fallback": false,
+                  "model_picker_enabled": true,
+                  "name": "Future Model v1",
+                  "object": "model",
+                  "preview": false,
+                  "vendor": "SomeNewVendor",
+                  "version": "v1.0"
+                }
+              ],
+              "object": "list"
+            }"#;
+
+        let schema: ModelSchema = serde_json::from_str(json).unwrap();
+
+        assert_eq!(schema.data.len(), 1);
+        assert_eq!(schema.data[0].id, "future-model-v1");
+        assert_eq!(schema.data[0].vendor, ModelVendor::Unknown);
+    }
 }
