@@ -36,7 +36,7 @@ impl GoDebugAdapter {
         delegate: &Arc<dyn DapDelegate>,
     ) -> Result<AdapterVersion> {
         let release = latest_github_release(
-            &"zed-industries/delve-shim-dap",
+            "zed-industries/delve-shim-dap",
             true,
             false,
             delegate.http_client(),
