@@ -90,11 +90,8 @@ impl ExampleInstance {
         worktrees_dir: &Path,
         repetition: usize,
     ) -> Self {
-        let name = thread.meta().name.to_string();
-        let run_directory = run_dir
-            .join(&name)
-            .join(repetition.to_string())
-            .to_path_buf();
+        let name = thread.meta().name;
+        let run_directory = run_dir.join(&name).join(repetition.to_string());
 
         let repo_path = repo_path_for_url(repos_dir, &thread.meta().url);
 
@@ -376,11 +373,10 @@ impl ExampleInstance {
             );
             let result = this.thread.conversation(&mut example_cx).await;
 
-            if let Err(err) = result {
-                if !err.is::<FailedAssertion>() {
+            if let Err(err) = result
+                && !err.is::<FailedAssertion>() {
                     return Err(err);
                 }
-            }
 
             println!("{}Stopped", this.log_prefix);
 
@@ -459,8 +455,8 @@ impl ExampleInstance {
         let mut output_file =
             File::create(self.run_directory.join("judge.md")).expect("failed to create judge.md");
 
-        let diff_task = self.judge_diff(model.clone(), &run_output, cx);
-        let thread_task = self.judge_thread(model.clone(), &run_output, cx);
+        let diff_task = self.judge_diff(model.clone(), run_output, cx);
+        let thread_task = self.judge_thread(model.clone(), run_output, cx);
 
         let (diff_result, thread_result) = futures::join!(diff_task, thread_task);
 
@@ -661,7 +657,7 @@ pub fn wait_for_lang_server(
         .update(cx, |buffer, cx| {
             lsp_store.update(cx, |lsp_store, cx| {
                 lsp_store
-                    .language_servers_for_local_buffer(&buffer, cx)
+                    .language_servers_for_local_buffer(buffer, cx)
                     .next()
                     .is_some()
             })
@@ -679,8 +675,8 @@ pub fn wait_for_lang_server(
         [
             cx.subscribe(&lsp_store, {
                 let log_prefix = log_prefix.clone();
-                move |_, event, _| match event {
-                    project::LspStoreEvent::LanguageServerUpdate {
+                move |_, event, _| {
+                    if let project::LspStoreEvent::LanguageServerUpdate {
                         message:
                             client::proto::update_language_server::Variant::WorkProgress(
                                 LspWorkProgress {
@@ -689,11 +685,13 @@ pub fn wait_for_lang_server(
                                 },
                             ),
                         ..
-                    } => println!("{}⟲ {message}", log_prefix),
-                    _ => {}
+                    } = event
+                    {
+                        println!("{}⟲ {message}", log_prefix)
+                    }
                 }
             }),
-            cx.subscribe(&project, {
+            cx.subscribe(project, {
                 let buffer = buffer.clone();
                 move |project, event, cx| match event {
                     project::Event::LanguageServerAdded(_, _, _) => {
@@ -771,7 +769,7 @@ pub async fn query_lsp_diagnostics(
 }
 
 fn parse_assertion_result(response: &str) -> Result<RanAssertionResult> {
-    let analysis = get_tag("analysis", response)?.to_string();
+    let analysis = get_tag("analysis", response)?;
     let passed = match get_tag("passed", response)?.to_lowercase().as_str() {
         "true" => true,
         "false" => false,
@@ -838,7 +836,7 @@ fn messages_to_markdown<'a>(message_iter: impl IntoIterator<Item = &'a Message>)
         for segment in &message.segments {
             match segment {
                 MessageSegment::Text(text) => {
-                    messages.push_str(&text);
+                    messages.push_str(text);
                     messages.push_str("\n\n");
                 }
                 MessageSegment::Thinking { text, signature } => {
@@ -846,7 +844,7 @@ fn messages_to_markdown<'a>(message_iter: impl IntoIterator<Item = &'a Message>)
                     if let Some(sig) = signature {
                         messages.push_str(&format!("Signature: {}\n\n", sig));
                     }
-                    messages.push_str(&text);
+                    messages.push_str(text);
                     messages.push_str("\n");
                 }
                 MessageSegment::RedactedThinking(items) => {
@@ -878,7 +876,7 @@ pub async fn send_language_model_request(
     request: LanguageModelRequest,
     cx: &AsyncApp,
 ) -> anyhow::Result<String> {
-    match model.stream_completion_text(request, &cx).await {
+    match model.stream_completion_text(request, cx).await {
         Ok(mut stream) => {
             let mut full_response = String::new();
             while let Some(chunk_result) = stream.stream.next().await {
@@ -915,9 +913,9 @@ impl RequestMarkdown {
             for tool in &request.tools {
                 write!(&mut tools, "# {}\n\n", tool.name).unwrap();
                 write!(&mut tools, "{}\n\n", tool.description).unwrap();
-                write!(
+                writeln!(
                     &mut tools,
-                    "{}\n",
+                    "{}",
                     MarkdownCodeBlock {
                         tag: "json",
                         text: &format!("{:#}", tool.input_schema)
@@ -1191,7 +1189,7 @@ mod test {
             output.analysis,
             Some("The model did a good job but there were still compilations errors.".into())
         );
-        assert_eq!(output.passed, true);
+        assert!(output.passed);
 
         let response = r#"
             Text around ignored
@@ -1211,6 +1209,6 @@ mod test {
             output.analysis,
             Some("Failed to compile:\n- Error 1\n- Error 2".into())
         );
-        assert_eq!(output.passed, false);
+        assert!(!output.passed);
     }
 }
