@@ -531,7 +531,6 @@ async fn test_extension_store(cx: &mut TestAppContext) {
 // `let fake_server = fake_servers.next().await.unwrap();`.
 // Reenable this test when we figure out why.
 #[gpui::test]
-#[cfg_attr(target_os = "windows", ignore)]
 async fn test_extension_store_with_test_extension(cx: &mut TestAppContext) {
     init_test(cx);
     cx.executor().allow_parking();
@@ -546,7 +545,7 @@ async fn test_extension_store_with_test_extension(cx: &mut TestAppContext) {
     let test_extension_dir = root_dir.join("extensions").join(test_extension_id);
 
     let fs = Arc::new(RealFs::new(None, cx.executor()));
-    let extensions_dir = TempTree::new(json!({
+    let extensions_tree = TempTree::new(json!({
         "installed": {},
         "work": {}
     }));
@@ -554,7 +553,7 @@ async fn test_extension_store_with_test_extension(cx: &mut TestAppContext) {
         "test.gleam": ""
     }));
 
-    let extensions_dir = extensions_dir.path().canonicalize().unwrap();
+    let extensions_dir = extensions_tree.path().canonicalize().unwrap();
     let project_dir = project_dir.path().canonicalize().unwrap();
 
     let project = Project::test(fs.clone(), [project_dir.as_path()], cx).await;
@@ -618,6 +617,10 @@ async fn test_extension_store_with_test_extension(cx: &mut TestAppContext) {
                                     {
                                         "name": format!("gleam-{version}-aarch64-unknown-linux-musl.tar.gz"),
                                         "browser_download_url": asset_download_uri
+                                    },
+                                    {
+                                        "name": format!("gleam-{version}-x86_64-pc-windows-msvc.tar.gz"),
+                                        "browser_download_url": asset_download_uri
                                     }
                                 ]
                             }
@@ -714,13 +717,17 @@ async fn test_extension_store_with_test_extension(cx: &mut TestAppContext) {
         .await
         .unwrap();
 
-    // todo(windows)
-    // This test hangs here on Windows.
     let fake_server = fake_servers.next().await.unwrap();
-    let expected_server_path =
-        extensions_dir.join(format!("work/{test_extension_id}/gleam-v1.2.3/gleam"));
+    let work_dir = extensions_dir.join(format!("work/{test_extension_id}"));
+    let expected_server_path = work_dir.join("gleam-v1.2.3/gleam");
     let expected_binary_contents = language_server_version.lock().binary_contents.clone();
 
+    // check that IO operations in extension work correctly
+    assert!(work_dir.join("dir-created-with-rel-path").exists());
+    assert!(work_dir.join("dir-created-with-abs-path").exists());
+    assert!(work_dir.join("file-created-with-abs-path").exists());
+    assert!(work_dir.join("file-created-with-rel-path").exists());
+
     assert_eq!(fake_server.binary.path, expected_server_path);
     assert_eq!(fake_server.binary.arguments, [OsString::from("lsp")]);
     assert_eq!(
@@ -826,7 +833,9 @@ async fn test_extension_store_with_test_extension(cx: &mut TestAppContext) {
     // Reload the extension, clearing its cache.
     // Start a new instance of the language server.
     extension_store
-        .update(cx, |store, cx| store.reload(Some("gleam".into()), cx))
+        .update(cx, |store, cx| {
+            store.reload(Some("test-extension".into()), cx)
+        })
         .await;
     cx.executor().run_until_parked();
     project.update(cx, |project, cx| {
