@@ -1,71 +1,44 @@
+use super::{latest, since_v0_1_0};
 use crate::wasm_host::WasmState;
 use anyhow::Result;
-use extension::{KeyValueStoreDelegate, ProjectDelegate, WorktreeDelegate};
+use extension::WorktreeDelegate;
 use gpui::BackgroundExecutor;
 use semantic_version::SemanticVersion;
 use std::sync::{Arc, OnceLock};
 use wasmtime::component::{Linker, Resource};
 
-use super::latest;
-
-pub const MIN_VERSION: SemanticVersion = SemanticVersion::new(0, 4, 0);
+pub const MIN_VERSION: SemanticVersion = SemanticVersion::new(0, 0, 6);
 
 wasmtime::component::bindgen!({
     async: true,
     trappable_imports: true,
-    path: "../extension_api/wit/since_v0.4.0",
+    path: "../extension_api/wit/since_v0.0.6",
     with: {
-        "worktree": ExtensionWorktree,
-        "project": ExtensionProject,
-        "key-value-store": ExtensionKeyValueStore,
-        "zed:extension/common": latest::zed::extension::common,
-        "zed:extension/github": latest::zed::extension::github,
-        "zed:extension/http-client": latest::zed::extension::http_client,
-        "zed:extension/lsp": latest::zed::extension::lsp,
-        "zed:extension/nodejs": latest::zed::extension::nodejs,
-        "zed:extension/platform": latest::zed::extension::platform,
-        "zed:extension/process": latest::zed::extension::process,
-        "zed:extension/slash-command": latest::zed::extension::slash_command,
+         "worktree": ExtensionWorktree,
+         "zed:extension/github": latest::zed::extension::github,
+         "zed:extension/lsp": since_v0_1_0::zed::extension::lsp,
+         "zed:extension/nodejs": latest::zed::extension::nodejs,
+         "zed:extension/platform": latest::zed::extension::platform,
     },
 });
 
 mod settings {
-    include!(concat!(env!("OUT_DIR"), "/since_v0.4.0/settings.rs"));
+    include!(concat!(env!("OUT_DIR"), "/since_v0.0.6/settings.rs"));
 }
 
 pub type ExtensionWorktree = Arc<dyn WorktreeDelegate>;
-pub type ExtensionProject = Arc<dyn ProjectDelegate>;
-pub type ExtensionKeyValueStore = Arc<dyn KeyValueStoreDelegate>;
 
 pub fn linker(executor: &BackgroundExecutor) -> &'static Linker<WasmState> {
     static LINKER: OnceLock<Linker<WasmState>> = OnceLock::new();
     LINKER.get_or_init(|| super::new_linker(executor, Extension::add_to_linker))
 }
 
-impl From<CodeLabel> for latest::CodeLabel {
-    fn from(value: CodeLabel) -> Self {
+impl From<Command> for latest::Command {
+    fn from(value: Command) -> Self {
         Self {
-            code: value.code,
-            spans: value.spans.into_iter().map(Into::into).collect(),
-            filter_range: value.filter_range,
-        }
-    }
-}
-
-impl From<CodeLabelSpan> for latest::CodeLabelSpan {
-    fn from(value: CodeLabelSpan) -> Self {
-        match value {
-            CodeLabelSpan::CodeRange(range) => Self::CodeRange(range),
-            CodeLabelSpan::Literal(literal) => Self::Literal(literal.into()),
-        }
-    }
-}
-
-impl From<CodeLabelSpanLiteral> for latest::CodeLabelSpanLiteral {
-    fn from(value: CodeLabelSpanLiteral) -> Self {
-        Self {
-            text: value.text,
-            highlight_name: value.highlight_name,
+            command: value.command,
+            args: value.args,
+            env: value.env,
         }
     }
 }
@@ -101,33 +74,40 @@ impl From<DownloadedFileType> for latest::DownloadedFileType {
     }
 }
 
-impl HostKeyValueStore for WasmState {
-    async fn insert(
-        &mut self,
-        kv_store: Resource<ExtensionKeyValueStore>,
-        key: String,
-        value: String,
-    ) -> wasmtime::Result<Result<(), String>> {
-        latest::HostKeyValueStore::insert(self, kv_store, key, value).await
-    }
-
-    async fn drop(&mut self, _worktree: Resource<ExtensionKeyValueStore>) -> Result<()> {
-        // We only ever hand out borrows of key-value stores.
-        Ok(())
+impl From<Range> for latest::Range {
+    fn from(value: Range) -> Self {
+        Self {
+            start: value.start,
+            end: value.end,
+        }
     }
 }
 
-impl HostProject for WasmState {
-    async fn worktree_ids(
-        &mut self,
-        project: Resource<ExtensionProject>,
-    ) -> wasmtime::Result<Vec<u64>> {
-        latest::HostProject::worktree_ids(self, project).await
+impl From<CodeLabelSpan> for latest::CodeLabelSpan {
+    fn from(value: CodeLabelSpan) -> Self {
+        match value {
+            CodeLabelSpan::CodeRange(range) => Self::CodeRange(range.into()),
+            CodeLabelSpan::Literal(literal) => Self::Literal(literal.into()),
+        }
     }
+}
 
-    async fn drop(&mut self, _project: Resource<Project>) -> Result<()> {
-        // We only ever hand out borrows of projects.
-        Ok(())
+impl From<CodeLabelSpanLiteral> for latest::CodeLabelSpanLiteral {
+    fn from(value: CodeLabelSpanLiteral) -> Self {
+        Self {
+            text: value.text,
+            highlight_name: value.highlight_name,
+        }
+    }
+}
+
+impl From<CodeLabel> for latest::CodeLabel {
+    fn from(value: CodeLabel) -> Self {
+        Self {
+            code: value.code,
+            spans: value.spans.into_iter().map(Into::into).collect(),
+            filter_range: value.filter_range.into(),
+        }
     }
 }
 
