@@ -1,12 +1,17 @@
-use futures::channel::oneshot;
-use futures::{FutureExt, select_biased};
-use gpui::{App, Context, Global, Subscription, Task, Window};
+mod flags;
+
 use std::cell::RefCell;
 use std::rc::Rc;
 use std::sync::LazyLock;
 use std::time::Duration;
 use std::{future::Future, pin::Pin, task::Poll};
 
+use futures::channel::oneshot;
+use futures::{FutureExt, select_biased};
+use gpui::{App, Context, Global, Subscription, Task, Window};
+
+pub use flags::*;
+
 #[derive(Default)]
 struct FeatureFlags {
     flags: Vec<String>,
@@ -23,7 +28,7 @@ impl FeatureFlags {
             return true;
         }
 
-        if self.staff && T::enabled_for_staff() {
+        if (cfg!(debug_assertions) || self.staff) && !*ZED_DISABLE_STAFF && T::enabled_for_staff() {
             return true;
         }
 
@@ -56,64 +61,6 @@ pub trait FeatureFlag {
     }
 }
 
-pub struct PredictEditsRateCompletionsFeatureFlag;
-impl FeatureFlag for PredictEditsRateCompletionsFeatureFlag {
-    const NAME: &'static str = "predict-edits-rate-completions";
-}
-
-pub struct LlmClosedBetaFeatureFlag {}
-impl FeatureFlag for LlmClosedBetaFeatureFlag {
-    const NAME: &'static str = "llm-closed-beta";
-}
-
-pub struct ZedProFeatureFlag {}
-impl FeatureFlag for ZedProFeatureFlag {
-    const NAME: &'static str = "zed-pro";
-}
-
-pub struct NotebookFeatureFlag;
-
-impl FeatureFlag for NotebookFeatureFlag {
-    const NAME: &'static str = "notebooks";
-}
-
-pub struct PanicFeatureFlag;
-
-impl FeatureFlag for PanicFeatureFlag {
-    const NAME: &'static str = "panic";
-}
-
-pub struct JjUiFeatureFlag {}
-
-impl FeatureFlag for JjUiFeatureFlag {
-    const NAME: &'static str = "jj-ui";
-}
-
-pub struct GeminiAndNativeFeatureFlag;
-
-impl FeatureFlag for GeminiAndNativeFeatureFlag {
-    // This was previously called "acp".
-    //
-    // We renamed it because existing builds used it to enable the Claude Code
-    // integration too, and we'd like to turn Gemini/Native on in new builds
-    // without enabling Claude Code in old builds.
-    const NAME: &'static str = "gemini-and-native";
-
-    fn enabled_for_all() -> bool {
-        true
-    }
-}
-
-pub struct ClaudeCodeFeatureFlag;
-
-impl FeatureFlag for ClaudeCodeFeatureFlag {
-    const NAME: &'static str = "claude-code";
-
-    fn enabled_for_all() -> bool {
-        true
-    }
-}
-
 pub trait FeatureFlagViewExt<V: 'static> {
     fn observe_flag<T: FeatureFlag, F>(&mut self, window: &Window, callback: F) -> Subscription
     where
@@ -209,7 +156,10 @@ impl FeatureFlagAppExt for App {
     fn has_flag<T: FeatureFlag>(&self) -> bool {
         self.try_global::<FeatureFlags>()
             .map(|flags| flags.has_flag::<T>())
-            .unwrap_or(T::enabled_for_all())
+            .unwrap_or_else(|| {
+                (cfg!(debug_assertions) && T::enabled_for_staff() && !*ZED_DISABLE_STAFF)
+                    || T::enabled_for_all()
+            })
     }
 
     fn is_staff(&self) -> bool {
