@@ -1,7 +1,7 @@
 use crate::file_finder_settings::FileFinderSettings;
 use file_icons::FileIcons;
 use futures::channel::oneshot;
-use fuzzy::{StringMatch, StringMatchCandidate};
+use fuzzy::{CharBag, StringMatch, StringMatchCandidate};
 use gpui::{HighlightStyle, StyledText, Task};
 use picker::{Picker, PickerDelegate};
 use project::{DirectoryItem, DirectoryLister};
@@ -23,7 +23,6 @@ use workspace::Workspace;
 
 pub(crate) struct OpenPathPrompt;
 
-#[derive(Debug)]
 pub struct OpenPathDelegate {
     tx: Option<oneshot::Sender<Option<Vec<PathBuf>>>>,
     lister: DirectoryLister,
@@ -35,6 +34,9 @@ pub struct OpenPathDelegate {
     prompt_root: String,
     path_style: PathStyle,
     replace_prompt: Task<()>,
+    render_footer:
+        Arc<dyn Fn(&mut Window, &mut Context<Picker<Self>>) -> Option<AnyElement> + 'static>,
+    hidden_entries: bool,
 }
 
 impl OpenPathDelegate {
@@ -60,9 +62,25 @@ impl OpenPathDelegate {
             },
             path_style,
             replace_prompt: Task::ready(()),
+            render_footer: Arc::new(|_, _| None),
+            hidden_entries: false,
         }
     }
 
+    pub fn with_footer(
+        mut self,
+        footer: Arc<
+            dyn Fn(&mut Window, &mut Context<Picker<Self>>) -> Option<AnyElement> + 'static,
+        >,
+    ) -> Self {
+        self.render_footer = footer;
+        self
+    }
+
+    pub fn show_hidden(mut self) -> Self {
+        self.hidden_entries = true;
+        self
+    }
     fn get_entry(&self, selected_match_index: usize) -> Option<CandidateInfo> {
         match &self.directory_state {
             DirectoryState::List { entries, .. } => {
@@ -125,6 +143,13 @@ impl OpenPathDelegate {
             DirectoryState::None { .. } => Vec::new(),
         }
     }
+
+    fn current_dir(&self) -> &'static str {
+        match self.path_style {
+            PathStyle::Posix => "./",
+            PathStyle::Windows => ".\\",
+        }
+    }
 }
 
 #[derive(Debug)]
@@ -233,6 +258,7 @@ impl PickerDelegate for OpenPathDelegate {
         cx: &mut Context<Picker<Self>>,
     ) -> Task<()> {
         let lister = &self.lister;
+        let input_is_empty = query.is_empty();
         let (dir, suffix) = get_dir_and_suffix(query, self.path_style);
 
         let query = match &self.directory_state {
@@ -261,8 +287,9 @@ impl PickerDelegate for OpenPathDelegate {
         self.cancel_flag.store(true, atomic::Ordering::Release);
         self.cancel_flag = Arc::new(AtomicBool::new(false));
         let cancel_flag = self.cancel_flag.clone();
-
+        let hidden_entries = self.hidden_entries;
         let parent_path_is_root = self.prompt_root == dir;
+        let current_dir = self.current_dir();
         cx.spawn_in(window, async move |this, cx| {
             if let Some(query) = query {
                 let paths = query.await;
@@ -353,10 +380,39 @@ impl PickerDelegate for OpenPathDelegate {
                 return;
             };
 
-            if !suffix.starts_with('.') {
-                new_entries.retain(|entry| !entry.path.string.starts_with('.'));
+            let mut max_id = 0;
+            if !suffix.starts_with('.') && !hidden_entries {
+                new_entries.retain(|entry| {
+                    max_id = max_id.max(entry.path.id);
+                    !entry.path.string.starts_with('.')
+                });
             }
+
             if suffix.is_empty() {
+                let should_prepend_with_current_dir = this
+                    .read_with(cx, |picker, _| {
+                        !input_is_empty
+                            && match &picker.delegate.directory_state {
+                                DirectoryState::List { error, .. } => error.is_none(),
+                                DirectoryState::Create { .. } => false,
+                                DirectoryState::None { .. } => false,
+                            }
+                    })
+                    .unwrap_or(false);
+                if should_prepend_with_current_dir {
+                    new_entries.insert(
+                        0,
+                        CandidateInfo {
+                            path: StringMatchCandidate {
+                                id: max_id + 1,
+                                string: current_dir.to_string(),
+                                char_bag: CharBag::from(current_dir),
+                            },
+                            is_dir: true,
+                        },
+                    );
+                }
+
                 this.update(cx, |this, cx| {
                     this.delegate.selected_index = 0;
                     this.delegate.string_matches = new_entries
@@ -485,6 +541,10 @@ impl PickerDelegate for OpenPathDelegate {
         _: &mut Context<Picker<Self>>,
     ) -> Option<String> {
         let candidate = self.get_entry(self.selected_index)?;
+        if candidate.path.string.is_empty() || candidate.path.string == self.current_dir() {
+            return None;
+        }
+
         let path_style = self.path_style;
         Some(
             maybe!({
@@ -629,14 +689,21 @@ impl PickerDelegate for OpenPathDelegate {
             DirectoryState::None { .. } => Vec::new(),
         };
 
+        let is_current_dir_candidate = candidate.path.string == self.current_dir();
+
         let file_icon = maybe!({
             if !settings.file_icons {
                 return None;
             }
+
+            let path = path::Path::new(&candidate.path.string);
             let icon = if candidate.is_dir {
-                FileIcons::get_folder_icon(false, cx)?
+                if is_current_dir_candidate {
+                    return Some(Icon::new(IconName::ReplyArrowRight).color(Color::Muted));
+                } else {
+                    FileIcons::get_folder_icon(false, path, cx)?
+                }
             } else {
-                let path = path::Path::new(&candidate.path.string);
                 FileIcons::get_icon(path, cx)?
             };
             Some(Icon::from_path(icon).color(Color::Muted))
@@ -652,6 +719,8 @@ impl PickerDelegate for OpenPathDelegate {
                     .child(HighlightedLabel::new(
                         if parent_path == &self.prompt_root {
                             format!("{}{}", self.prompt_root, candidate.path.string)
+                        } else if is_current_dir_candidate {
+                            "open this directory".to_string()
                         } else {
                             candidate.path.string
                         },
@@ -732,6 +801,14 @@ impl PickerDelegate for OpenPathDelegate {
         }
     }
 
+    fn render_footer(
+        &self,
+        window: &mut Window,
+        cx: &mut Context<Picker<Self>>,
+    ) -> Option<AnyElement> {
+        (self.render_footer)(window, cx)
+    }
+
     fn no_matches_text(&self, _window: &mut Window, _cx: &mut App) -> Option<SharedString> {
         Some(match &self.directory_state {
             DirectoryState::Create { .. } => SharedString::from("Type a path…"),
@@ -747,6 +824,17 @@ impl PickerDelegate for OpenPathDelegate {
     fn placeholder_text(&self, _window: &mut Window, _cx: &mut App) -> Arc<str> {
         Arc::from(format!("[directory{MAIN_SEPARATOR_STR}]filename.ext"))
     }
+
+    fn separators_after_indices(&self) -> Vec<usize> {
+        let Some(m) = self.string_matches.first() else {
+            return Vec::new();
+        };
+        if m.string == self.current_dir() {
+            vec![0]
+        } else {
+            Vec::new()
+        }
+    }
 }
 
 fn path_candidates(
