@@ -63,15 +63,15 @@ mod tests {
     #[test]
     fn test_parse_git_diff_name_status() {
         let input = concat!(
-            "M Cargo.lock ",
-            "M crates/project/Cargo.toml ",
-            "M crates/project/src/buffer_store.rs ",
-            "D crates/project/src/git.rs ",
-            "A crates/project/src/git_store.rs ",
-            "A crates/project/src/git_store/git_traversal.rs ",
-            "M crates/project/src/project.rs ",
-            "M crates/project/src/worktree_store.rs ",
-            "M crates/project_panel/src/project_panel.rs ",
+            "M\x00Cargo.lock\x00",
+            "M\x00crates/project/Cargo.toml\x00",
+            "M\x00crates/project/src/buffer_store.rs\x00",
+            "D\x00crates/project/src/git.rs\x00",
+            "A\x00crates/project/src/git_store.rs\x00",
+            "A\x00crates/project/src/git_store/git_traversal.rs\x00",
+            "M\x00crates/project/src/project.rs\x00",
+            "M\x00crates/project/src/worktree_store.rs\x00",
+            "M\x00crates/project_panel/src/project_panel.rs\x00",
         );
 
         let output = parse_git_diff_name_status(input).collect::<Vec<_>>();
