@@ -957,15 +957,17 @@ impl Clipboard {
         }
         // At this point we know that the clipboard does not exist.
         let ctx = Arc::new(Inner::new()?);
-        let join_handle;
-        {
-            let ctx = Arc::clone(&ctx);
-            join_handle = std::thread::spawn(move || {
-                if let Err(error) = serve_requests(ctx) {
-                    log::error!("Worker thread errored with: {}", error);
+        let join_handle = std::thread::Builder::new()
+            .name("Clipboard".to_owned())
+            .spawn({
+                let ctx = Arc::clone(&ctx);
+                move || {
+                    if let Err(error) = serve_requests(ctx) {
+                        log::error!("Worker thread errored with: {}", error);
+                    }
                 }
-            });
-        }
+            })
+            .unwrap();
         *global_cb = Some(GlobalClipboard {
             inner: Arc::clone(&ctx),
             server_handle: join_handle,
