@@ -2,6 +2,7 @@ mod clipboard;
 mod destination_list;
 mod direct_write;
 mod directx_atlas;
+mod directx_devices;
 mod directx_renderer;
 mod dispatcher;
 mod display;
@@ -18,6 +19,7 @@ pub(crate) use clipboard::*;
 pub(crate) use destination_list::*;
 pub(crate) use direct_write::*;
 pub(crate) use directx_atlas::*;
+pub(crate) use directx_devices::*;
 pub(crate) use directx_renderer::*;
 pub(crate) use dispatcher::*;
 pub(crate) use display::*;
