@@ -146,6 +146,7 @@ pub enum IconName {
     Library,
     LineHeight,
     ListCollapse,
+    ListFilter,
     ListTodo,
     ListTree,
     ListX,
@@ -262,6 +263,7 @@ pub enum IconName {
     ZedPredictError,
     ZedPredictUp,
     ZedXCopilot,
+    Linux,
 }
 
 impl IconName {
