@@ -14,7 +14,10 @@ use language::{
     DiagnosticSeverity, LanguageServerId, Point, ToOffset as _, ToPoint as _,
 };
 use project::lsp_store::CompletionDocumentation;
-use project::{Completion, CompletionResponse, CompletionSource, Project, ProjectPath};
+use project::{
+    Completion, CompletionDisplayOptions, CompletionResponse, CompletionSource, Project,
+    ProjectPath,
+};
 use std::fmt::Write as _;
 use std::ops::Range;
 use std::path::Path;
@@ -25,7 +28,7 @@ use util::split_str_with_ranges;
 
 /// Path used for unsaved buffer that contains style json. To support the json language server, this
 /// matches the name used in the generated schemas.
-const ZED_INSPECTOR_STYLE_JSON: &str = "/zed-inspector-style.json";
+const ZED_INSPECTOR_STYLE_JSON: &str = util_macros::path!("/zed-inspector-style.json");
 
 pub(crate) struct DivInspector {
     state: State,
@@ -93,8 +96,8 @@ impl DivInspector {
                     Ok((json_style_buffer, rust_style_buffer)) => {
                         this.update_in(cx, |this, window, cx| {
                             this.state = State::BuffersLoaded {
-                                json_style_buffer: json_style_buffer,
-                                rust_style_buffer: rust_style_buffer,
+                                json_style_buffer,
+                                rust_style_buffer,
                             };
 
                             // Initialize editors immediately instead of waiting for
@@ -200,8 +203,8 @@ impl DivInspector {
         cx.subscribe_in(&json_style_editor, window, {
             let id = id.clone();
             let rust_style_buffer = rust_style_buffer.clone();
-            move |this, editor, event: &EditorEvent, window, cx| match event {
-                EditorEvent::BufferEdited => {
+            move |this, editor, event: &EditorEvent, window, cx| {
+                if event == &EditorEvent::BufferEdited {
                     let style_json = editor.read(cx).text(cx);
                     match serde_json_lenient::from_str_lenient::<StyleRefinement>(&style_json) {
                         Ok(new_style) => {
@@ -243,7 +246,6 @@ impl DivInspector {
                         Err(err) => this.json_style_error = Some(err.to_string().into()),
                     }
                 }
-                _ => {}
             }
         })
         .detach();
@@ -251,11 +253,10 @@ impl DivInspector {
         cx.subscribe(&rust_style_editor, {
             let json_style_buffer = json_style_buffer.clone();
             let rust_style_buffer = rust_style_buffer.clone();
-            move |this, _editor, event: &EditorEvent, cx| match event {
-                EditorEvent::BufferEdited => {
+            move |this, _editor, event: &EditorEvent, cx| {
+                if let EditorEvent::BufferEdited = event {
                     this.update_json_style_from_rust(&json_style_buffer, &rust_style_buffer, cx);
                 }
-                _ => {}
             }
         })
         .detach();
@@ -271,23 +272,19 @@ impl DivInspector {
     }
 
     fn reset_style(&mut self, cx: &mut App) {
-        match &self.state {
-            State::Ready {
-                rust_style_buffer,
-                json_style_buffer,
-                ..
-            } => {
-                if let Err(err) = self.reset_style_editors(
-                    &rust_style_buffer.clone(),
-                    &json_style_buffer.clone(),
-                    cx,
-                ) {
-                    self.json_style_error = Some(format!("{err}").into());
-                } else {
-                    self.json_style_error = None;
-                }
+        if let State::Ready {
+            rust_style_buffer,
+            json_style_buffer,
+            ..
+        } = &self.state
+        {
+            if let Err(err) =
+                self.reset_style_editors(&rust_style_buffer.clone(), &json_style_buffer.clone(), cx)
+            {
+                self.json_style_error = Some(format!("{err}").into());
+            } else {
+                self.json_style_error = None;
             }
-            _ => {}
         }
     }
 
@@ -395,11 +392,11 @@ impl DivInspector {
             .zip(self.rust_completion_replace_range.as_ref())
         {
             let before_text = snapshot
-                .text_for_range(0..completion_range.start.to_offset(&snapshot))
+                .text_for_range(0..completion_range.start.to_offset(snapshot))
                 .collect::<String>();
             let after_text = snapshot
                 .text_for_range(
-                    completion_range.end.to_offset(&snapshot)
+                    completion_range.end.to_offset(snapshot)
                         ..snapshot.clip_offset(usize::MAX, Bias::Left),
                 )
                 .collect::<String>();
@@ -670,6 +667,7 @@ impl CompletionProvider for RustStyleCompletionProvider {
                     confirm: None,
                 })
                 .collect(),
+            display_options: CompletionDisplayOptions::default(),
             is_incomplete: false,
         }]))
     }
@@ -702,10 +700,10 @@ impl CompletionProvider for RustStyleCompletionProvider {
 }
 
 fn completion_replace_range(snapshot: &BufferSnapshot, anchor: &Anchor) -> Option<Range<Anchor>> {
-    let point = anchor.to_point(&snapshot);
-    let offset = point.to_offset(&snapshot);
-    let line_start = Point::new(point.row, 0).to_offset(&snapshot);
-    let line_end = Point::new(point.row, snapshot.line_len(point.row)).to_offset(&snapshot);
+    let point = anchor.to_point(snapshot);
+    let offset = point.to_offset(snapshot);
+    let line_start = Point::new(point.row, 0).to_offset(snapshot);
+    let line_end = Point::new(point.row, snapshot.line_len(point.row)).to_offset(snapshot);
     let mut lines = snapshot.text_for_range(line_start..line_end).lines();
     let line = lines.next()?;
 
