@@ -264,13 +264,9 @@ pub(crate) fn render_ai_setup_page(
                     );
 
                     let fs = <dyn Fs>::global(cx);
-                    update_settings_file::<DisableAiSettings>(
-                        fs,
-                        cx,
-                        move |ai_settings: &mut Option<bool>, _| {
-                            *ai_settings = Some(enabled);
-                        },
-                    );
+                    update_settings_file::<DisableAiSettings>(fs, cx, move |ai_settings, _| {
+                        ai_settings.disable_ai = Some(enabled);
+                    });
                 },
             )
             .tab_index({
