@@ -119,7 +119,7 @@ impl Prettier {
                                             None
                                         }
                                     }).any(|workspace_definition| {
-                                        workspace_definition == subproject_path.to_string_lossy() || PathMatcher::new(&[workspace_definition]).ok().map_or(false, |path_matcher| path_matcher.is_match(subproject_path))
+                                        workspace_definition == subproject_path.to_string_lossy() || PathMatcher::new(&[workspace_definition]).ok().is_some_and(|path_matcher| path_matcher.is_match(subproject_path))
                                     }) {
                                         anyhow::ensure!(has_prettier_in_node_modules(fs, &path_to_check).await?, "Path {path_to_check:?} is the workspace root for project in {closest_package_json_path:?}, but it has no prettier installed");
                                         log::info!("Found prettier path {path_to_check:?} in the workspace root for project in {closest_package_json_path:?}");
@@ -185,11 +185,11 @@ impl Prettier {
                     .metadata(&ignore_path)
                     .await
                     .with_context(|| format!("fetching metadata for {ignore_path:?}"))?
+                    && !metadata.is_dir
+                    && !metadata.is_symlink
                 {
-                    if !metadata.is_dir && !metadata.is_symlink {
-                        log::info!("Found prettier ignore at {ignore_path:?}");
-                        return Ok(ControlFlow::Continue(Some(path_to_check)));
-                    }
+                    log::info!("Found prettier ignore at {ignore_path:?}");
+                    return Ok(ControlFlow::Continue(Some(path_to_check)));
                 }
                 match &closest_package_json_path {
                     None => closest_package_json_path = Some(path_to_check.clone()),
@@ -217,19 +217,19 @@ impl Prettier {
                                     workspace_definition == subproject_path.to_string_lossy()
                                         || PathMatcher::new(&[workspace_definition])
                                             .ok()
-                                            .map_or(false, |path_matcher| {
+                                            .is_some_and(|path_matcher| {
                                                 path_matcher.is_match(subproject_path)
                                             })
                                 })
                             {
                                 let workspace_ignore = path_to_check.join(".prettierignore");
-                                if let Some(metadata) = fs.metadata(&workspace_ignore).await? {
-                                    if !metadata.is_dir {
-                                        log::info!(
-                                            "Found prettier ignore at workspace root {workspace_ignore:?}"
-                                        );
-                                        return Ok(ControlFlow::Continue(Some(path_to_check)));
-                                    }
+                                if let Some(metadata) = fs.metadata(&workspace_ignore).await?
+                                    && !metadata.is_dir
+                                {
+                                    log::info!(
+                                        "Found prettier ignore at workspace root {workspace_ignore:?}"
+                                    );
+                                    return Ok(ControlFlow::Continue(Some(path_to_check)));
                                 }
                             }
                         }
@@ -549,18 +549,16 @@ async fn read_package_json(
         .metadata(&possible_package_json)
         .await
         .with_context(|| format!("fetching metadata for package json {possible_package_json:?}"))?
+        && !package_json_metadata.is_dir
+        && !package_json_metadata.is_symlink
     {
-        if !package_json_metadata.is_dir && !package_json_metadata.is_symlink {
-            let package_json_contents = fs
-                .load(&possible_package_json)
-                .await
-                .with_context(|| format!("reading {possible_package_json:?} file contents"))?;
-            return serde_json::from_str::<HashMap<String, serde_json::Value>>(
-                &package_json_contents,
-            )
+        let package_json_contents = fs
+            .load(&possible_package_json)
+            .await
+            .with_context(|| format!("reading {possible_package_json:?} file contents"))?;
+        return serde_json::from_str::<HashMap<String, serde_json::Value>>(&package_json_contents)
             .map(Some)
             .with_context(|| format!("parsing {possible_package_json:?} file contents"));
-        }
     }
     Ok(None)
 }
