@@ -1,7 +1,8 @@
 use anyhow::Result;
 use collections::HashMap;
 use gpui::{App, AppContext as _, Context, Entity, Task, WeakEntity};
-use itertools::Itertools;
+
+use itertools::Itertools as _;
 use language::LanguageName;
 use remote::RemoteClient;
 use settings::{Settings, SettingsLocation};
@@ -11,7 +12,7 @@ use std::{
     path::{Path, PathBuf},
     sync::Arc,
 };
-use task::{Shell, ShellBuilder, SpawnInTerminal};
+use task::{Shell, ShellBuilder, ShellKind, SpawnInTerminal};
 use terminal::{
     TaskState, TaskStatus, Terminal, TerminalBuilder, terminal_settings::TerminalSettings,
 };
@@ -48,14 +49,6 @@ impl Project {
         cx: &mut Context<Self>,
     ) -> Task<Result<Entity<Terminal>>> {
         let is_via_remote = self.remote_client.is_some();
-        let project_path_context = self
-            .active_entry()
-            .and_then(|entry_id| self.worktree_id_for_entry(entry_id, cx))
-            .or_else(|| self.visible_worktrees(cx).next().map(|wt| wt.read(cx).id()))
-            .map(|worktree_id| ProjectPath {
-                worktree_id,
-                path: Arc::from(Path::new("")),
-            });
 
         let path: Option<Arc<Path>> = if let Some(cwd) = &spawn_task.cwd {
             if is_via_remote {
@@ -123,61 +116,120 @@ impl Project {
             },
         };
 
-        let toolchain = project_path_context
+        let project_path_contexts = self
+            .active_entry()
+            .and_then(|entry_id| self.path_for_entry(entry_id, cx))
+            .into_iter()
+            .chain(
+                self.visible_worktrees(cx)
+                    .map(|wt| wt.read(cx).id())
+                    .map(|worktree_id| ProjectPath {
+                        worktree_id,
+                        path: Arc::from(Path::new("")),
+                    }),
+            );
+        let toolchains = project_path_contexts
             .filter(|_| detect_venv)
-            .map(|p| self.active_toolchain(p, LanguageName::new("Python"), cx));
+            .map(|p| self.active_toolchain(p, LanguageName::new("Python"), cx))
+            .collect::<Vec<_>>();
         let lang_registry = self.languages.clone();
         let fs = self.fs.clone();
         cx.spawn(async move |project, cx| {
+            let shell_kind = ShellKind::new(&shell);
             let activation_script = maybe!(async {
-                let toolchain = toolchain?.await?;
-                lang_registry
-                    .language_for_name(&toolchain.language_name.0)
-                    .await
-                    .ok()?
-                    .toolchain_lister()?
-                    .activation_script(&toolchain, fs.as_ref())
-                    .await
+                for toolchain in toolchains {
+                    let Some(toolchain) = toolchain.await else {
+                        continue;
+                    };
+                    let language = lang_registry
+                        .language_for_name(&toolchain.language_name.0)
+                        .await
+                        .ok();
+                    let lister = language?.toolchain_lister();
+                    return Some(
+                        lister?
+                            .activation_script(&toolchain, shell_kind, fs.as_ref())
+                            .await,
+                    );
+                }
+                None
             })
-            .await;
+            .await
+            .unwrap_or_default();
 
             project.update(cx, move |this, cx| {
-                let shell = {
+                let format_to_run = || {
+                    if let Some(command) = &spawn_task.command {
+                        let mut command: Option<Cow<str>> = shlex::try_quote(command).ok();
+                        if let Some(command) = &mut command
+                            && command.starts_with('"')
+                            && let Some(prefix) = shell_kind.command_prefix()
+                        {
+                            *command = Cow::Owned(format!("{prefix}{command}"));
+                        }
+
+                        let args = spawn_task
+                            .args
+                            .iter()
+                            .filter_map(|arg| shlex::try_quote(arg).ok());
+                        command.into_iter().chain(args).join(" ")
+                    } else {
+                        // todo: this breaks for remotes to windows
+                        format!("exec {shell} -l")
+                    }
+                };
+
+                let (shell, env) = {
                     env.extend(spawn_task.env);
                     match remote_client {
-                        Some(remote_client) => create_remote_shell(
-                            spawn_task
-                                .command
-                                .as_ref()
-                                .map(|command| (command, &spawn_task.args)),
-                            &mut env,
-                            path,
-                            remote_client,
-                            activation_script.clone(),
-                            cx,
-                        )?,
-                        None => match activation_script.clone() {
-                            Some(activation_script) => {
-                                let to_run = if let Some(command) = spawn_task.command {
-                                    let command: Option<Cow<str>> = shlex::try_quote(&command).ok();
-                                    let args = spawn_task
-                                        .args
-                                        .iter()
-                                        .filter_map(|arg| shlex::try_quote(arg).ok());
-                                    command.into_iter().chain(args).join(" ")
-                                } else {
-                                    format!("exec {shell} -l")
-                                };
-                                Shell::WithArguments {
-                                    program: get_default_system_shell(),
-                                    args: vec![
-                                        "-c".to_owned(),
-                                        format!("{activation_script}; {to_run}",),
-                                    ],
-                                    title_override: None,
-                                }
+                        Some(remote_client) => match activation_script.clone() {
+                            activation_script if !activation_script.is_empty() => {
+                                let activation_script = activation_script.join("; ");
+                                let to_run = format_to_run();
+                                let args =
+                                    vec!["-c".to_owned(), format!("{activation_script}; {to_run}")];
+                                create_remote_shell(
+                                    Some((
+                                        &remote_client
+                                            .read(cx)
+                                            .shell()
+                                            .unwrap_or_else(get_default_system_shell),
+                                        &args,
+                                    )),
+                                    env,
+                                    path,
+                                    remote_client,
+                                    cx,
+                                )?
                             }
-                            None => {
+                            _ => create_remote_shell(
+                                spawn_task
+                                    .command
+                                    .as_ref()
+                                    .map(|command| (command, &spawn_task.args)),
+                                env,
+                                path,
+                                remote_client,
+                                cx,
+                            )?,
+                        },
+                        None => match activation_script.clone() {
+                            activation_script if !activation_script.is_empty() => {
+                                let activation_script = activation_script.join("; ");
+                                let to_run = format_to_run();
+
+                                let arg = format!("{activation_script}; {to_run}");
+
+                                (
+                                    Shell::WithArguments {
+                                        program: shell,
+                                        args: vec!["-c".to_owned(), arg],
+                                        title_override: None,
+                                    },
+                                    env,
+                                )
+                            }
+                            _ => (
                                 if let Some(program) = spawn_task.command {
                                     Shell::WithArguments {
                                         program,
@@ -186,8 +238,9 @@ impl Project {
                                     }
                                 } else {
                                     Shell::System
-                                }
-                            }
+                                },
+                                env,
+                            ),
                         },
                     }
                 };
@@ -237,14 +290,6 @@ impl Project {
         cwd: Option<PathBuf>,
         cx: &mut Context<Self>,
     ) -> Task<Result<Entity<Terminal>>> {
-        let project_path_context = self
-            .active_entry()
-            .and_then(|entry_id| self.worktree_id_for_entry(entry_id, cx))
-            .or_else(|| self.visible_worktrees(cx).next().map(|wt| wt.read(cx).id()))
-            .map(|worktree_id| ProjectPath {
-                worktree_id,
-                path: Arc::from(Path::new("")),
-            });
         let path = cwd.map(|p| Arc::from(&*p));
         let is_via_remote = self.remote_client.is_some();
 
@@ -272,11 +317,24 @@ impl Project {
 
         let local_path = if is_via_remote { None } else { path.clone() };
 
-        let toolchain = project_path_context
+        let project_path_contexts = self
+            .active_entry()
+            .and_then(|entry_id| self.path_for_entry(entry_id, cx))
+            .into_iter()
+            .chain(
+                self.visible_worktrees(cx)
+                    .map(|wt| wt.read(cx).id())
+                    .map(|worktree_id| ProjectPath {
+                        worktree_id,
+                        path: Arc::from(Path::new("")),
+                    }),
+            );
+        let toolchains = project_path_contexts
             .filter(|_| detect_venv)
-            .map(|p| self.active_toolchain(p, LanguageName::new("Python"), cx));
+            .map(|p| self.active_toolchain(p, LanguageName::new("Python"), cx))
+            .collect::<Vec<_>>();
         let remote_client = self.remote_client.clone();
-        let shell = match &remote_client {
+        let shell_kind = ShellKind::new(&match &remote_client {
             Some(remote_client) => remote_client
                 .read(cx)
                 .shell()
@@ -290,43 +348,38 @@ impl Project {
                 } => program.clone(),
                 Shell::System => get_system_shell(),
             },
-        };
+        });
 
         let lang_registry = self.languages.clone();
         let fs = self.fs.clone();
         cx.spawn(async move |project, cx| {
             let activation_script = maybe!(async {
-                let toolchain = toolchain?.await?;
-                let language = lang_registry
-                    .language_for_name(&toolchain.language_name.0)
-                    .await
-                    .ok();
-                let lister = language?.toolchain_lister();
-                lister?.activation_script(&toolchain, fs.as_ref()).await
+                for toolchain in toolchains {
+                    let Some(toolchain) = toolchain.await else {
+                        continue;
+                    };
+                    let language = lang_registry
+                        .language_for_name(&toolchain.language_name.0)
+                        .await
+                        .ok();
+                    let lister = language?.toolchain_lister();
+                    return Some(
+                        lister?
+                            .activation_script(&toolchain, shell_kind, fs.as_ref())
+                            .await,
+                    );
+                }
+                None
             })
-            .await;
+            .await
+            .unwrap_or_default();
             project.update(cx, move |this, cx| {
-                let shell = {
+                let (shell, env) = {
                     match remote_client {
-                        Some(remote_client) => create_remote_shell(
-                            None,
-                            &mut env,
-                            path,
-                            remote_client,
-                            activation_script.clone(),
-                            cx,
-                        )?,
-                        None => match activation_script.clone() {
-                            Some(activation_script) => Shell::WithArguments {
-                                program: get_default_system_shell(),
-                                args: vec![
-                                    "-c".to_owned(),
-                                    format!("{activation_script}; exec {shell} -l",),
-                                ],
-                                title_override: Some(shell.into()),
-                            },
-                            None => settings.shell,
-                        },
+                        Some(remote_client) => {
+                            create_remote_shell(None, env, path, remote_client, cx)?
+                        }
+                        None => (settings.shell, env),
                     }
                 };
                 TerminalBuilder::new(
@@ -437,15 +490,10 @@ impl Project {
 
         match remote_client {
             Some(remote_client) => {
-                let command_template = remote_client.read(cx).build_command(
-                    Some(command),
-                    &args,
-                    &env,
-                    None,
-                    // todo
-                    None,
-                    None,
-                )?;
+                let command_template =
+                    remote_client
+                        .read(cx)
+                        .build_command(Some(command), &args, &env, None, None)?;
                 let mut command = std::process::Command::new(command_template.program);
                 command.args(command_template.args);
                 command.envs(command_template.env);
@@ -470,12 +518,11 @@ impl Project {
 
 fn create_remote_shell(
     spawn_command: Option<(&String, &Vec<String>)>,
-    env: &mut HashMap<String, String>,
+    mut env: HashMap<String, String>,
     working_directory: Option<Arc<Path>>,
     remote_client: Entity<RemoteClient>,
-    activation_script: Option<String>,
     cx: &mut App,
-) -> Result<Shell> {
+) -> Result<(Shell, HashMap<String, String>)> {
     // Alacritty sets its terminfo to `alacritty`, this requiring hosts to have it installed
     // to properly display colors.
     // We do not have the luxury of assuming the host has it installed,
@@ -491,19 +538,20 @@ fn create_remote_shell(
     let command = remote_client.read(cx).build_command(
         program,
         args.as_slice(),
-        env,
+        &env,
         working_directory.map(|path| path.display().to_string()),
-        activation_script,
         None,
     )?;
-    *env = command.env;
 
     log::debug!("Connecting to a remote server: {:?}", command.program);
-    let host = remote_client.read(cx).connection_options().host;
+    let host = remote_client.read(cx).connection_options().display_name();
 
-    Ok(Shell::WithArguments {
-        program: command.program,
-        args: command.args,
-        title_override: Some(format!("{} — Terminal", host).into()),
-    })
+    Ok((
+        Shell::WithArguments {
+            program: command.program,
+            args: command.args,
+            title_override: Some(format!("{} — Terminal", host).into()),
+        },
+        command.env,
+    ))
 }
