@@ -171,10 +171,10 @@ impl ReplStore {
                 .map(KernelSpecification::Jupyter)
                 .collect::<Vec<_>>();
 
-            if let Some(remote_task) = remote_kernel_specifications {
-                if let Ok(remote_specs) = remote_task.await {
-                    all_specs.extend(remote_specs);
-                }
+            if let Some(remote_task) = remote_kernel_specifications
+                && let Ok(remote_specs) = remote_task.await
+            {
+                all_specs.extend(remote_specs);
             }
 
             anyhow::Ok(all_specs)
