@@ -310,7 +310,7 @@ impl PickerDelegate for ScopeSelectorDelegate {
         _window: &mut Window,
         cx: &mut Context<Picker<Self>>,
     ) -> Option<Self::ListItem> {
-        let mat = &self.matches[ix];
+        let mat = &self.matches.get(ix)?;
         let name_label = mat.string.clone();
 
         let scope_name = ScopeName(Cow::Owned(
