@@ -1,12 +1,13 @@
 use std::path::PathBuf;
 
-use editor::{Editor, EditorEvent};
+use editor::Editor;
 use file_icons::FileIcons;
 use gpui::{
     App, Context, Entity, EventEmitter, FocusHandle, Focusable, ImageSource, IntoElement,
     ParentElement, Render, Resource, RetainAllImageCache, Styled, Subscription, WeakEntity, Window,
     div, img,
 };
+use multi_buffer::{Event as MultiBufferEvent, MultiBuffer};
 use ui::prelude::*;
 use workspace::item::Item;
 use workspace::{Pane, Workspace};
@@ -17,7 +18,7 @@ pub struct SvgPreviewView {
     focus_handle: FocusHandle,
     svg_path: Option<PathBuf>,
     image_cache: Entity<RetainAllImageCache>,
-    _editor_subscription: Subscription,
+    _buffer_subscription: Subscription,
     _workspace_subscription: Option<Subscription>,
 }
 
@@ -109,7 +110,7 @@ impl SvgPreviewView {
         editor: &Entity<Editor>,
         cx: &App,
     ) -> Option<usize> {
-        let editor_path = Self::get_svg_path(editor, cx);
+        let editor_path = Self::get_svg_path(editor.read(cx).buffer(), cx);
         pane.items_of_type::<SvgPreviewView>()
             .find(|view| {
                 let view_read = view.read(cx);
@@ -150,25 +151,10 @@ impl SvgPreviewView {
         cx: &mut Context<Workspace>,
     ) -> Entity<Self> {
         cx.new(|cx| {
-            let svg_path = Self::get_svg_path(&active_editor, cx);
             let image_cache = RetainAllImageCache::new(cx);
-
-            let subscription = cx.subscribe_in(
-                &active_editor,
-                window,
-                |this: &mut SvgPreviewView, _editor, event: &EditorEvent, window, cx| {
-                    if event == &EditorEvent::Saved {
-                        // Remove cached image to force reload
-                        if let Some(svg_path) = &this.svg_path {
-                            let resource = Resource::Path(svg_path.clone().into());
-                            this.image_cache.update(cx, |cache, cx| {
-                                cache.remove(&resource, window, cx);
-                            });
-                        }
-                        cx.notify();
-                    }
-                },
-            );
+            let buffer = active_editor.read(cx).buffer();
+            let svg_path = Self::get_svg_path(buffer, cx);
+            let subscription = Self::create_buffer_subscription(&buffer.clone(), window, cx);
 
             // Subscribe to workspace active item changes to follow SVG files
             let workspace_subscription = if mode == SvgPreviewMode::Follow {
@@ -179,17 +165,24 @@ impl SvgPreviewView {
                         |this: &mut SvgPreviewView,
                          workspace,
                          event: &workspace::Event,
-                         _window,
+                         window,
                          cx| {
                             if let workspace::Event::ActiveItemChanged = event {
                                 let workspace_read = workspace.read(cx);
                                 if let Some(active_item) = workspace_read.active_item(cx)
-                                    && let Some(editor_entity) = active_item.downcast::<Editor>()
-                                    && Self::is_svg_file(&editor_entity, cx)
+                                    && let Some(editor) = active_item.downcast::<Editor>()
+                                    && Self::is_svg_file(&editor, cx)
                                 {
-                                    let new_path = Self::get_svg_path(&editor_entity, cx);
+                                    let buffer = editor.read(cx).buffer();
+                                    let new_path = Self::get_svg_path(&buffer, cx);
                                     if this.svg_path != new_path {
                                         this.svg_path = new_path;
+                                        this._buffer_subscription =
+                                            Self::create_buffer_subscription(
+                                                &buffer.clone(),
+                                                window,
+                                                cx,
+                                            );
                                         cx.notify();
                                     }
                                 }
@@ -205,20 +198,44 @@ impl SvgPreviewView {
                 focus_handle: cx.focus_handle(),
                 svg_path,
                 image_cache,
-                _editor_subscription: subscription,
+                _buffer_subscription: subscription,
                 _workspace_subscription: workspace_subscription,
             }
         })
     }
 
-    pub fn is_svg_file<C>(editor: &Entity<Editor>, cx: &C) -> bool
-    where
-        C: std::borrow::Borrow<App>,
-    {
-        let app = cx.borrow();
-        let buffer = editor.read(app).buffer().read(app);
+    fn create_buffer_subscription(
+        active_buffer: &Entity<MultiBuffer>,
+        window: &mut Window,
+        cx: &mut Context<Self>,
+    ) -> Subscription {
+        cx.subscribe_in(
+            active_buffer,
+            window,
+            |this: &mut SvgPreviewView, buffer, event: &MultiBufferEvent, window, cx| {
+                let potential_path_change = event == &MultiBufferEvent::FileHandleChanged;
+                if event == &MultiBufferEvent::Saved || potential_path_change {
+                    // Remove cached image to force reload
+                    if let Some(svg_path) = &this.svg_path {
+                        let resource = Resource::Path(svg_path.clone().into());
+                        this.image_cache.update(cx, |cache, cx| {
+                            cache.remove(&resource, window, cx);
+                        });
+                    }
+
+                    if potential_path_change {
+                        this.svg_path = Self::get_svg_path(buffer, cx);
+                    }
+                    cx.notify();
+                }
+            },
+        )
+    }
+
+    pub fn is_svg_file(editor: &Entity<Editor>, cx: &App) -> bool {
+        let buffer = editor.read(cx).buffer().read(cx);
         if let Some(buffer) = buffer.as_singleton()
-            && let Some(file) = buffer.read(app).file()
+            && let Some(file) = buffer.read(cx).file()
         {
             return file
                 .path()
@@ -230,15 +247,11 @@ impl SvgPreviewView {
         false
     }
 
-    fn get_svg_path<C>(editor: &Entity<Editor>, cx: &C) -> Option<PathBuf>
-    where
-        C: std::borrow::Borrow<App>,
-    {
-        let app = cx.borrow();
-        let buffer = editor.read(app).buffer().read(app).as_singleton()?;
-        let file = buffer.read(app).file()?;
+    fn get_svg_path(buffer: &Entity<MultiBuffer>, cx: &App) -> Option<PathBuf> {
+        let buffer = buffer.read(cx).as_singleton()?;
+        let file = buffer.read(cx).file()?;
         let local_file = file.as_local()?;
-        Some(local_file.abs_path(app))
+        Some(local_file.abs_path(cx))
     }
 }
 
