@@ -148,9 +148,9 @@ pub fn toggle_modal(
 ) -> Task<()> {
     let task_store = workspace.project().read(cx).task_store().clone();
     let workspace_handle = workspace.weak_handle();
-    let can_open_modal = workspace.project().update(cx, |project, cx| {
-        project.is_local() || project.ssh_connection_string(cx).is_some() || project.is_via_ssh()
-    });
+    let can_open_modal = workspace
+        .project()
+        .read_with(cx, |project, _| !project.is_via_collab());
     if can_open_modal {
         let task_contexts = task_contexts(workspace, window, cx);
         cx.spawn_in(window, async move |workspace, cx| {
@@ -227,10 +227,10 @@ where
 
                 tasks.retain_mut(|(task_source_kind, target_task)| {
                     if predicate((task_source_kind, target_task)) {
-                        if let Some(overrides) = &overrides {
-                            if let Some(target_override) = overrides.reveal_target {
-                                target_task.reveal_target = target_override;
-                            }
+                        if let Some(overrides) = &overrides
+                            && let Some(target_override) = overrides.reveal_target
+                        {
+                            target_task.reveal_target = target_override;
                         }
                         workspace.schedule_task(
                             task_source_kind.clone(),
@@ -283,7 +283,7 @@ pub fn task_contexts(
                 .project()
                 .read(cx)
                 .worktree_for_id(*worktree_id, cx)
-                .map_or(false, |worktree| is_visible_directory(&worktree, cx))
+                .is_some_and(|worktree| is_visible_directory(&worktree, cx))
         })
         .or_else(|| {
             workspace
@@ -343,11 +343,10 @@ pub fn task_contexts(
         task_contexts.lsp_task_sources = lsp_task_sources;
         task_contexts.latest_selection = latest_selection;
 
-        if let Some(editor_context_task) = editor_context_task {
-            if let Some(editor_context) = editor_context_task.await {
-                task_contexts.active_item_context =
-                    Some((active_worktree, location, editor_context));
-            }
+        if let Some(editor_context_task) = editor_context_task
+            && let Some(editor_context) = editor_context_task.await
+        {
+            task_contexts.active_item_context = Some((active_worktree, location, editor_context));
         }
 
         if let Some(active_worktree) = active_worktree {
@@ -373,7 +372,7 @@ pub fn task_contexts(
 
 fn is_visible_directory(worktree: &Entity<Worktree>, cx: &App) -> bool {
     let worktree = worktree.read(cx);
-    worktree.is_visible() && worktree.root_entry().map_or(false, |entry| entry.is_dir())
+    worktree.is_visible() && worktree.root_entry().is_some_and(|entry| entry.is_dir())
 }
 
 fn worktree_context(worktree_abs_path: &Path) -> TaskContext {
@@ -435,7 +434,7 @@ mod tests {
         )
         .await;
         let project = Project::test(fs, [path!("/dir").as_ref()], cx).await;
-        let worktree_store = project.read_with(cx, |project, _| project.worktree_store().clone());
+        let worktree_store = project.read_with(cx, |project, _| project.worktree_store());
         let rust_language = Arc::new(
             Language::new(
                 LanguageConfig::default(),
