@@ -107,6 +107,8 @@ pub fn init(themes_to_load: LoadThemes, cx: &mut App) {
     let mut prev_buffer_font_size_settings =
         ThemeSettings::get_global(cx).buffer_font_size_settings();
     let mut prev_ui_font_size_settings = ThemeSettings::get_global(cx).ui_font_size_settings();
+    let mut prev_agent_font_size_settings =
+        ThemeSettings::get_global(cx).agent_font_size_settings();
     cx.observe_global::<SettingsStore>(move |cx| {
         let buffer_font_size_settings = ThemeSettings::get_global(cx).buffer_font_size_settings();
         if buffer_font_size_settings != prev_buffer_font_size_settings {
@@ -119,6 +121,12 @@ pub fn init(themes_to_load: LoadThemes, cx: &mut App) {
             prev_ui_font_size_settings = ui_font_size_settings;
             reset_ui_font_size(cx);
         }
+
+        let agent_font_size_settings = ThemeSettings::get_global(cx).agent_font_size_settings();
+        if agent_font_size_settings != prev_agent_font_size_settings {
+            prev_agent_font_size_settings = agent_font_size_settings;
+            reset_agent_font_size(cx);
+        }
     })
     .detach();
 }
@@ -249,9 +257,9 @@ pub fn refine_theme_family(theme_family_content: ThemeFamilyContent) -> ThemeFam
     let author = theme_family_content.author.clone();
 
     let mut theme_family = ThemeFamily {
-        id: id.clone(),
-        name: name.clone().into(),
-        author: author.clone().into(),
+        id,
+        name: name.into(),
+        author: author.into(),
         themes: vec![],
         scales: default_color_scales(),
     };
