@@ -175,7 +175,7 @@ impl Tooltip {
         move |_, cx| {
             let title = title.clone();
             cx.new(|_| Self {
-                title: title,
+                title,
                 meta: None,
                 key_binding: None,
             })
