@@ -443,7 +443,7 @@ impl NeovimConnection {
             }
             Mode::Insert | Mode::Normal | Mode::Replace => selections
                 .push(Point::new(selection_row, selection_col)..Point::new(cursor_row, cursor_col)),
-            Mode::HelixNormal => unreachable!(),
+            Mode::HelixNormal | Mode::HelixSelect => unreachable!(),
         }
 
         let ranges = encode_ranges(&text, &selections);
